DIGIT                       [0-9]
HEXDIGIT                    [0-9a-f]
HEXVAL                      0{HEXDIGIT}{8}
LETTER                      [a-zA-Z_.]
ID                          {LETTER}({LETTER}|{DIGIT})*
QUOTED_ID                   \"[^\"]+\"
UINT                        {DIGIT}+
INTEG                       {UINT}[B]?
FP                          {DIGIT}+[.]{DIGIT}+
FLOAT                       {FP}([e]{INTEG})?

%{

#include <vector>
#include <map>
#include <string>
#include <string.h>
#include "Ast.h"


using namespace std;

extern int expparser_linenr;
int expparser_parse();
//map<string, const RttTgenType *> typemap;
extern Expression *expression;

/* Include der Parser Token Konstanten und String Bibliothek */
#include "expparser_yacc.h"

%}

/* Lexer Optionen festlegen */
%option noyywrap nounput

%%

"int"                         { return(INT_T); }
"bool"                        { return(BOOL_T); }


"true"                        { return(TRUE); }
"false"                       { return(FALSE); }

"<="			              { return(LE_OP); }
">="			              { return(GE_OP); }
"=="			              { return(EQ_OP); }
"!="			              { return(NE_OP); }
"and"			              { return(AND_OP); }
"&&"			              { return(AND_OP); }
"or"			              { return(OR_OP); }
"||"			              { return(OR_OP); }
"xor"			              { return(XOR_OP); }
"not"                                 { return('!'); }
"="                                   { fprintf(stderr,"\nEncountered illegal '=' - did you mean '=='?"); return(ILLEGAL); }

"(" | 
")" |
"[" |
"]" |
"{" |
"}" |
"<" |
">" |
";" |
":" |
"." |
"," |
"+" |
"-" |
"!" |
"/" |
"&" |
"\"" |
"*" | 
"@" |
"%"                           { return(*expparser_text); }


{ID}                          { expparser_lval.cp = strdup(expparser_text); return(IDENT); }
{QUOTED_ID}                   { expparser_text[strlen(expparser_text)-1] = 0;
                                expparser_lval.cp = strdup(expparser_text+1); 
                                return(IDENT); }
{INTEG}                       { if ( expparser_text[strlen(expparser_text)-1] == 'B' )
                                       expparser_text[strlen(expparser_text)-1] = 0;
                                expparser_lval.ll = atol(expparser_text); 
                                return(INTEGER); }
{FLOAT}                       { expparser_lval.doub = atof(expparser_text); return(FLOATPT); }

[ \t\r]                         /* Ignoriere sonstigen Whitespace. */

[\n]                          { expparser_linenr++; }
.                             { return(ILLEGAL); }


%%

Expression *parseExp(char *c) {

    yy_scan_string(c);

    /* Bei erfolgreichem parsen den Syntaxbaum ausgeben */
    if(expparser_parse() == 0) {
        yy_delete_buffer(YY_CURRENT_BUFFER);
       // typemap.clear();
        return expression;
    } else {
        yy_delete_buffer(YY_CURRENT_BUFFER);
       // typemap.clear();
        return NULL;
    }
}



// Local Variables:
// coding: utf-8
// mode: c++
// c-basic-offset: 4
// End:
