
%{


  /* Includes für IO, Syntaxbaum, und Symboltabelle */


#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <map>

#include "Ast.h"

#define PRINTDEBUG 0

  /* Schnittstelle zum Lexer und Fehlerbehandlung */
  extern FILE *expparser_in;
  extern int expparser_lex();
  void expparser_error(char *);

  int expparser_linenr = 1;

  void test_debug(string s);

  Expression *expression;
  //extern map<string, const RttTgenType *> typemap;

  %}

/* Union mit allen möglichen Datentypen der Attribute von Symbolen */
%union {
  char *cp;
  double doub;
  long long ll;
  int i;
  bool b;
  class Expression *expr;
  vector<Expression *> *expvec;
  class RttTgenStmt *stmt;
  vector<RttTgenStmt *> *stmtvec;
  class RttTgenBlock *blk;
};

 
/* Token für Schlüsselwörter*/

%token<cp> IDENT
%token<doub> FLOATPT
%token<ll> INTEGER

%token ILLEGAL
%token TRUE FALSE BOOL_T INT_T
%token LE_OP GE_OP EQ_OP NE_OP AND_OP OR_OP XOR_OP

%type<expr> primary_expression postfix_expression unary_expression cast_expression
%type<expr> multiplicative_expression additive_expression shift_expression relational_expression
%type<expr> equality_expression and_expression exclusive_or_expression inclusive_or_expression
%type<expr> logical_and_expression logical_or_expression
%type<i> unary_operator
%type<expvec> argument_expression_list

%start exp

%%

primary_expression
: IDENT
{ 
    $$ = new Identifier($1);
}
| TRUE 
{ 
    $$ = new Literal("1");
}
| FALSE 
{ 
    $$ = new Literal("0");
}
| INTEGER 
{  
    ostringstream oss;
    oss << $1;
    $$ = new Literal(oss.str());
}
| FLOATPT 
{  
    ostringstream oss;
    oss << $1;
    $$ = new Literal(oss.str());
}
| '(' logical_or_expression ')' 
{ 
    $$ = $2;
}
;

postfix_expression
: primary_expression 
{ 
    $$ = $1;
}
| postfix_expression '[' logical_or_expression ']'
{ 
  $$ = $1;
  // @todo
}
| postfix_expression '(' ')'
{ 
    NaryExpression *e = new NaryExpression($1->toString());
    $$ = e;
}
| postfix_expression '(' argument_expression_list ')'
{ 
    NaryExpression *e = new NaryExpression($1->toString());
    for ( vector<Expression*>::iterator
	      p = $3->begin();
	  p != $3->end();
	  p++ ) {
	e->addOperand(*p);
    }
    $$ = e;
}
;

argument_expression_list
: logical_or_expression 
{  
  $$ = new vector<Expression*>(); 
  $$->push_back($1);
}
| argument_expression_list ',' logical_or_expression 
{  
  $$ = $1; 
  $$->push_back($3);
}
;

unary_expression
: postfix_expression 
{  
    $$ = $1;
}
| unary_operator cast_expression
{
    string opstr;
    switch($1) {
    case '+':
	opstr = "+";
	break;
    case '-':
	opstr = "-";
	break;
    case '!':
	opstr = "!";
	break;
    default:
	opstr = "***UNKNOWN_OPERATOR***";
	break;
    }

    NaryExpression *e = new NaryExpression(opstr);
    e->addOperand($2);
    $$ = e;
}
;

unary_operator
: '+' 
{  
    $$ = '+';
}
| '-' 
{  
     $$ = '-';
}
| '!' 
{  
     $$ = '!';
}
;

cast_expression
: unary_expression 
{ 
    $$ = $1;
}
;


multiplicative_expression
: cast_expression 
{  
    $$ = $1;
}
| multiplicative_expression '*' cast_expression
{
    NaryExpression *e = new NaryExpression("*");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e; 
}
| multiplicative_expression '/' cast_expression
{
    NaryExpression *e = new NaryExpression("/");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;  
}
| multiplicative_expression '%' cast_expression
{
    NaryExpression *e = new NaryExpression("%");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e; 
}
;


additive_expression
: multiplicative_expression { }
| additive_expression '+' multiplicative_expression
{
    NaryExpression *e = new NaryExpression("+");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;  
}
| additive_expression '-' multiplicative_expression
{
    NaryExpression *e = new NaryExpression("-");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;   
}
;


shift_expression
: additive_expression 
{  
    $$ = $1;
}
;


relational_expression
: shift_expression { }
| relational_expression '<' shift_expression
{
    NaryExpression *e = new NaryExpression("<");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e; 
}
| relational_expression '>' shift_expression
{
    NaryExpression *e = new NaryExpression(">");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e; 
}
| relational_expression LE_OP shift_expression
{
    NaryExpression *e = new NaryExpression("<=");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e; 
}
| relational_expression GE_OP shift_expression
{
    NaryExpression *e = new NaryExpression(">=");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e; 
}
;


equality_expression
: relational_expression {  }
| equality_expression EQ_OP relational_expression
{
    NaryExpression *e = new NaryExpression("==");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;
}
| equality_expression NE_OP relational_expression
{
    NaryExpression *e = new NaryExpression("!=");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;
}
;


and_expression
: equality_expression 
{  
    $$ = $1;
}
;


exclusive_or_expression
: and_expression 
{ 
  $$ = $1;
}
;


inclusive_or_expression
: exclusive_or_expression 
{  
  $$ = $1;
}
;


logical_and_expression
: inclusive_or_expression 
{ 
  $$ = $1
}
| logical_and_expression AND_OP inclusive_or_expression
{
    NaryExpression *e = new NaryExpression("&&");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;
}
;


logical_or_expression
: logical_and_expression 
{ 
  $$ = $1; 
}
| logical_or_expression OR_OP logical_and_expression
{
    NaryExpression *e = new NaryExpression("||");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;
}
| logical_or_expression XOR_OP logical_and_expression
{
    NaryExpression *e = new NaryExpression("xor");
    e->addOperand($1);
    e->addOperand($3);
    $$ = e;
}

exp: logical_or_expression 
{ 
    expression = $1; 
}
;



%%

  /* Funktion für Fehlerbehandlung */
void expparser_error(char *msg)
{
  cerr << endl << "ERROR: Syntax error in line " << expparser_linenr << ":" << msg << endl;
}



void test_debug(string s){

  if(PRINTDEBUG)
    cout << endl << s;

}

// Local Variables:
// coding: utf-8
// mode: c++
// c-basic-offset: 4
// End:
